/***************************************************************************
 *   Copyright (C) 2008 by Justin L. Boss                                  *
 *   justin@rtechguys.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "seacreature.h"
#include <QtDebug>
#include <QFile>

SeaCreature::SeaCreature (const char filename[])
{
	RX = 0;
	RY = 0;
	RZ = 0;
	TailPathCount=qrand()%56;
	Size = 1;
	TailSpan = 1;
	Mammal = false;

	Matrix = new GLfloat[16];
	OldMtx = new GLfloat[16];
	RotX   = new GLfloat[16];
	RotY   = new GLfloat[16];
	RotZ   = new GLfloat[16];
	Tran   = new GLfloat[16];

	ifstream aglfile (filename);

	aglfile >> BoneSize;

	BonMtx = new GLfloat *[BoneSize];
	for (int i=0; i<BoneSize; i++)
		BonMtx[i] = new GLfloat[16];

	Bones = new GLfloat[BoneSize];
	HalfBone = BoneSize / 2;
	for (int i=0;i<BoneSize-1;i++)
		aglfile >> Bones[i];

	aglfile >> MeshSize;

	TransMesh = new MeshData[MeshSize];

	StaticMesh = new SMeshData[MeshSize];

	for (int i=0;i<MeshSize;i++)
	{
		aglfile >> StaticMesh[i].vx;
		aglfile >> StaticMesh[i].vy;
		aglfile >> StaticMesh[i].vz;
		aglfile >> StaticMesh[i].nx;
		aglfile >> StaticMesh[i].ny;
		aglfile >> StaticMesh[i].nz;
		aglfile >> StaticMesh[i].bv;
		aglfile >> StaticMesh[i].bi;
	}

	aglfile >> UVCordSize;
	UVCord = new UVData[UVCordSize];
	for (int i=0;i<UVCordSize;i++)
	{
		aglfile >> UVCord[i].x;
		aglfile >> UVCord[i].y;
	}

	aglfile >> FaceSize;
	Faces = new FaceData[FaceSize];
	for (int i=0;i<FaceSize;i++)
	{
		aglfile >> Faces[i].a;
		aglfile >> Faces[i].b;
		aglfile >> Faces[i].c;
		aglfile >> Faces[i].d;
	}
	aglfile.close();

	UpdateTail();
	Time.start();
	LastTime = Time.elapsed();
}

SeaCreature::~SeaCreature()
{
	delete Matrix;
	delete OldMtx;
	delete RotX;
	delete RotY;
	delete RotZ;
	delete Tran;

	delete Bones;

	for (int i=0; i<BoneSize; i++)
		delete [] BonMtx[i];
	delete [] BonMtx;

	delete TransMesh;

	delete StaticMesh;

	delete UVCord;

	delete Faces;
}

void SeaCreature::SetTailSpan(int span)
{
	TailSpan = span * 0.1;
}

void SeaCreature::SetSize(int size)
{
	Size = size * 0.1;
}

void SeaCreature::LoadIdentity()
{
	Matrix[0] = 1.0; Matrix[4] = 0.0; Matrix[8] = 0.0; Matrix[12] = 0.0;
	Matrix[1] = 0.0; Matrix[5] = 1.0; Matrix[9] = 0.0; Matrix[13] = 0.0;
	Matrix[2] = 0.0; Matrix[6] = 0.0; Matrix[10]= 1.0; Matrix[14] = 0.0;
	Matrix[3] = 0.0; Matrix[7] = 0.0; Matrix[11]= 0.0; Matrix[15] = 1.0;

	RotX[0] = 1.0; RotX[4] = 0.0; RotX[8]  = 0.0; RotX[12] = 0.0;
	RotX[1] = 0.0; RotX[5] = 1.0; RotX[9]  = 0.0; RotX[13] = 0.0;
	RotX[2] = 0.0; RotX[6] = 0.0; RotX[10] = 1.0; RotX[14] = 0.0;
	RotX[3] = 0.0; RotX[7] = 0.0; RotX[11] = 0.0; RotX[15] = 1.0;

	RotY[0] = 1.0; RotY[4] = 0.0; RotY[8]  = 0.0; RotY[12] = 0.0;
	RotY[1] = 0.0; RotY[5] = 1.0; RotY[9]  = 0.0; RotY[13] = 0.0;
	RotY[2] = 0.0; RotY[6] = 0.0; RotY[10] = 1.0; RotY[14] = 0.0;
	RotY[3] = 0.0; RotY[7] = 0.0; RotY[11] = 0.0; RotY[15] = 1.0;

	RotZ[0] = 1.0; RotZ[4] = 0.0; RotZ[8]  = 0.0; RotZ[12] = 0.0;
	RotZ[1] = 0.0; RotZ[5] = 1.0; RotZ[9]  = 0.0; RotZ[13] = 0.0;
	RotZ[2] = 0.0; RotZ[6] = 0.0; RotZ[10] = 1.0; RotZ[14] = 0.0;
	RotZ[3] = 0.0; RotZ[7] = 0.0; RotZ[11] = 0.0; RotZ[15] = 1.0;

	Tran[0] = 1.0; Tran[4] = 0.0; Tran[8]  = 0.0; Tran[12] = 0.0;
	Tran[1] = 0.0; Tran[5] = 1.0; Tran[9]  = 0.0; Tran[13] = 0.0;
	Tran[2] = 0.0; Tran[6] = 0.0; Tran[10] = 1.0; Tran[14] = 0.0;
	Tran[3] = 0.0; Tran[7] = 0.0; Tran[11] = 0.0; Tran[15] = 1.0;
}


void SeaCreature::CatMtx (GLfloat *newmtx)
{
	OldMtx[0]  = Matrix[0] * newmtx[0]  + Matrix[4] * newmtx[1]  + Matrix[8]  * newmtx[2]  + Matrix[12] * newmtx[3];
	OldMtx[1]  = Matrix[1] * newmtx[0]  + Matrix[5] * newmtx[1]  + Matrix[9]  * newmtx[2]  + Matrix[13] * newmtx[3];
	OldMtx[2]  = Matrix[2] * newmtx[0]  + Matrix[6] * newmtx[1]  + Matrix[10] * newmtx[2]  + Matrix[14] * newmtx[3];
	OldMtx[3]  = Matrix[3] * newmtx[0]  + Matrix[7] * newmtx[1]  + Matrix[11] * newmtx[2]  + Matrix[15] * newmtx[3];
	OldMtx[4]  = Matrix[0] * newmtx[4]  + Matrix[4] * newmtx[5]  + Matrix[8]  * newmtx[6]  + Matrix[12] * newmtx[7];
	OldMtx[5]  = Matrix[1] * newmtx[4]  + Matrix[5] * newmtx[5]  + Matrix[9]  * newmtx[6]  + Matrix[13] * newmtx[7];
	OldMtx[6]  = Matrix[2] * newmtx[4]  + Matrix[6] * newmtx[5]  + Matrix[10] * newmtx[6]  + Matrix[14] * newmtx[7];
	OldMtx[7]  = Matrix[3] * newmtx[4]  + Matrix[7] * newmtx[5]  + Matrix[11] * newmtx[6]  + Matrix[15] * newmtx[7];
	OldMtx[8]  = Matrix[0] * newmtx[8]  + Matrix[4] * newmtx[9]  + Matrix[8]  * newmtx[10] + Matrix[12] * newmtx[11];
	OldMtx[9]  = Matrix[1] * newmtx[8]  + Matrix[5] * newmtx[9]  + Matrix[9]  * newmtx[10] + Matrix[13] * newmtx[11];
	OldMtx[10] = Matrix[2] * newmtx[8]  + Matrix[6] * newmtx[9]  + Matrix[10] * newmtx[10] + Matrix[14] * newmtx[11];
	OldMtx[11] = Matrix[3] * newmtx[8]  + Matrix[7] * newmtx[9]  + Matrix[11] * newmtx[10] + Matrix[15] * newmtx[11];
	OldMtx[12] = Matrix[0] * newmtx[12] + Matrix[4] * newmtx[13] + Matrix[8]  * newmtx[14] + Matrix[12] * newmtx[15];
	OldMtx[13] = Matrix[1] * newmtx[12] + Matrix[5] * newmtx[13] + Matrix[9]  * newmtx[14] + Matrix[13] * newmtx[15];
	OldMtx[14] = Matrix[2] * newmtx[12] + Matrix[6] * newmtx[13] + Matrix[10] * newmtx[14] + Matrix[14] * newmtx[15];
	OldMtx[15] = Matrix[3] * newmtx[12] + Matrix[7] * newmtx[13] + Matrix[11] * newmtx[14] + Matrix[15] * newmtx[15];
	GLfloat * temp = OldMtx;
	OldMtx = Matrix;
	Matrix = temp;
}


void SeaCreature::Draw (GLfloat offset, int speed, GLfloat height)
{
	glBindTexture (GL_TEXTURE_2D, TextureName);

	gsBegin ((Time.elapsed() - LastTime) * speed * 0.05);
	glPushMatrix();

	if (RX)
		glRotatef (RX, 1.0, 0.0, 0.0);
	if (RY)
		glRotatef (RY, 0.0, 1.0, 0.0);
	if (RZ)
		glRotatef (RZ, 0.0, 0.0, 1.0);

	if (Mammal)
		glRotatef (TailPath[int(TailPathCount)%56][0]*0.7, 0.0, 0.0, 1.0);

	glBegin (GL_QUADS);
	for (int x=0; x<FaceSize; x++)
	{
		glTexCoord2f (UVCord[Faces[x].a].x, UVCord[Faces[x].a].y);
		glNormal3f (TransMesh[Faces[x].a].nx, TransMesh[Faces[x].a].ny, TransMesh[Faces[x].a].nz);
		glVertex3f (TransMesh[Faces[x].a].vx-offset, TransMesh[Faces[x].a].vy + height, TransMesh[Faces[x].a].vz);

		glTexCoord2f (UVCord[Faces[x].b].x, UVCord[Faces[x].b].y);
		glNormal3f (TransMesh[Faces[x].b].nx, TransMesh[Faces[x].b].ny, TransMesh[Faces[x].b].nz);
		glVertex3f (TransMesh[Faces[x].b].vx-offset, TransMesh[Faces[x].b].vy + height, TransMesh[Faces[x].b].vz);

		glTexCoord2f (UVCord[Faces[x].c].x, UVCord[Faces[x].c].y);
		glNormal3f (TransMesh[Faces[x].c].nx,TransMesh[Faces[x].c].ny,TransMesh[Faces[x].c].nz);
		glVertex3f (TransMesh[Faces[x].c].vx-offset,TransMesh[Faces[x].c].vy + height, TransMesh[Faces[x].c].vz);

		glTexCoord2f (UVCord[Faces[x].d].x, UVCord[Faces[x].d].y);
		glNormal3f (TransMesh[Faces[x].d].nx,TransMesh[Faces[x].d].ny,TransMesh[Faces[x].d].nz);
		glVertex3f (TransMesh[Faces[x].d].vx-offset,TransMesh[Faces[x].d].vy + height, TransMesh[Faces[x].d].vz);
	}
	glEnd();
	glPopMatrix();
	gsEnd();

	TailPathCount += (Time.elapsed() - LastTime) * speed * 0.001;
	LastTime = Time.elapsed();

	UpdateTail();
}

void SeaCreature::Draw (GLfloat offset, int speed, GLfloat gsMatrix[])
{
	glBindTexture (GL_TEXTURE_2D, TextureName);

	gsBegin ((Time.elapsed() - LastTime) * speed * 0.05);

	glPushMatrix();

	glLoadMatrixf (gsMatrix);
	if (RX)
		glRotatef (RX, 1.0, 0.0, 0.0);
	if (RY)
		glRotatef (RY, 0.0, 1.0, 0.0);
	if (RZ)
		glRotatef (RZ, 0.0, 0.0, 1.0);

	if (Mammal)
		glRotatef (TailPath[int(TailPathCount)%56][0]*0.7, 0.0, 0.0, 1.0);

	glBegin (GL_QUADS);
	for (int x=0; x<FaceSize; x++)
	{
		glTexCoord2f (UVCord[Faces[x].a].x, UVCord[Faces[x].a].y);
		glNormal3f (TransMesh[Faces[x].a].nx, TransMesh[Faces[x].a].ny, TransMesh[Faces[x].a].nz);
		glVertex3f (TransMesh[Faces[x].a].vx-offset, TransMesh[Faces[x].a].vy, TransMesh[Faces[x].a].vz);

		glTexCoord2f (UVCord[Faces[x].b].x, UVCord[Faces[x].b].y);
		glNormal3f (TransMesh[Faces[x].b].nx, TransMesh[Faces[x].b].ny, TransMesh[Faces[x].b].nz);
		glVertex3f (TransMesh[Faces[x].b].vx-offset, TransMesh[Faces[x].b].vy, TransMesh[Faces[x].b].vz);

		glTexCoord2f (UVCord[Faces[x].c].x, UVCord[Faces[x].c].y);
		glNormal3f (TransMesh[Faces[x].c].nx,TransMesh[Faces[x].c].ny,TransMesh[Faces[x].c].nz);
		glVertex3f (TransMesh[Faces[x].c].vx-offset,TransMesh[Faces[x].c].vy,TransMesh[Faces[x].c].vz);

		glTexCoord2f (UVCord[Faces[x].d].x, UVCord[Faces[x].d].y);
		glNormal3f (TransMesh[Faces[x].d].nx,TransMesh[Faces[x].d].ny,TransMesh[Faces[x].d].nz);
		glVertex3f (TransMesh[Faces[x].d].vx-offset,TransMesh[Faces[x].d].vy,TransMesh[Faces[x].d].vz);
	}
	glEnd();
	glPopMatrix();//
	gsEnd();

	TailPathCount += (Time.elapsed() - LastTime) * speed * 0.001;
	LastTime = Time.elapsed();
	UpdateTail();
}

void SeaCreature::Rotate (GLfloat X, GLfloat Y, GLfloat Z)
{
	if (X)
	{
		RotX[5]= cos(X*PI_180);
		RotX[9]= sin(X*PI_180);
		RotX[6]= -RotX[9];
		RotX[10]= RotX[5];
		CatMtx (RotX);
	}
	if (Y)
	{
		RotY[0]= cos(Y*PI_180);
		RotY[2]= sin(Y*PI_180);
		RotY[8]= -RotY[2];
		RotY[10]= RotY[0];
		CatMtx (RotY);
	}
	if (Z)
	{
		RotZ[0]= cos (Z*PI_180);
		RotZ[4]= sin (Z*PI_180);
		RotZ[1]=-RotZ[4];
		RotZ[5]= RotZ[0];
		CatMtx (RotZ);
	}
}

void SeaCreature::Translate (GLfloat TX, GLfloat TY, GLfloat TZ)
{
	Tran[12]=TX;
	Tran[13]=TY;
	Tran[14]=TZ;
	CatMtx (Tran);
}

void SeaCreature::Scale (GLfloat SX, GLfloat SY, GLfloat SZ)
{
	Matrix[0] =SX;
	Matrix[5] =SY;
	Matrix[10]=SZ;
}

void SeaCreature::SetTexture (GLuint name)
{
	TextureName = name;
}

void SeaCreature::UpdateTail()						// New faster one.
{
        LoadIdentity();
	Scale (Size, Size, Size);
	CpyMtx (BonMtx[0]);
	for (int i = 1; i < BoneSize; i++)
	{
		Translate (Bones[i-1], 0.0 ,0.0);
		if (Mammal)
			Rotate (0, 0, TailPath[int(TailPathCount)%56][int((i-1)/HalfBone)]*TailSpan);
		else
			Rotate (0, TailPath[int(TailPathCount)%56][int((i-1)/HalfBone)]*TailSpan, 0);
		CpyMtx (BonMtx[i]);
	}
	for (int x=0; x < MeshSize; x++)
	{
		TransMesh[x].vx = StaticMesh[x].vx * BonMtx[StaticMesh[x].bi][0] + StaticMesh[x].vy * BonMtx[StaticMesh[x].bi][4] + StaticMesh[x].vz * BonMtx[StaticMesh[x].bi] [8] + BonMtx[StaticMesh[x].bi][12];
		TransMesh[x].vy = StaticMesh[x].vx * BonMtx[StaticMesh[x].bi][1] + StaticMesh[x].vy * BonMtx[StaticMesh[x].bi][5] + StaticMesh[x].vz * BonMtx[StaticMesh[x].bi] [9] + BonMtx[StaticMesh[x].bi][13];
		TransMesh[x].vz = StaticMesh[x].vx * BonMtx[StaticMesh[x].bi][2] + StaticMesh[x].vy * BonMtx[StaticMesh[x].bi][6] + StaticMesh[x].vz * BonMtx[StaticMesh[x].bi][10] + BonMtx[StaticMesh[x].bi][14];
		TransMesh[x].nx = StaticMesh[x].nx * BonMtx[StaticMesh[x].bi][0] + StaticMesh[x].ny * BonMtx[StaticMesh[x].bi][4] + StaticMesh[x].nz * BonMtx[StaticMesh[x].bi] [8];
		TransMesh[x].ny = StaticMesh[x].nx * BonMtx[StaticMesh[x].bi][1] + StaticMesh[x].ny * BonMtx[StaticMesh[x].bi][5] + StaticMesh[x].nz * BonMtx[StaticMesh[x].bi] [9];
		TransMesh[x].nz = StaticMesh[x].nx * BonMtx[StaticMesh[x].bi][2] + StaticMesh[x].ny * BonMtx[StaticMesh[x].bi][6] + StaticMesh[x].nz * BonMtx[StaticMesh[x].bi][10];
	}
}

void SeaCreature::CpyMtx(GLfloat *destmatrix)
{
	destmatrix [0]=Matrix [0]; destmatrix [1]=Matrix [1]; destmatrix [2]=Matrix [2]; destmatrix [3]=Matrix [3];
	destmatrix [4]=Matrix [4]; destmatrix [5]=Matrix [5]; destmatrix [6]=Matrix [6]; destmatrix [7]=Matrix [7];
	destmatrix [8]=Matrix [8]; destmatrix [9]=Matrix [9]; destmatrix[10]=Matrix[10]; destmatrix[11]=Matrix[11];
	destmatrix[12]=Matrix[12]; destmatrix[13]=Matrix[13]; destmatrix[14]=Matrix[14]; destmatrix[15]=Matrix[15];
}

void SeaCreature::scRotate(GLfloat X, GLfloat Y, GLfloat Z)
{
	RX = X;
	RY = Y;
	RZ = Z;
}
